require 'sinatra'
require 'sinatra/json'
require 'protobuf'
require 'require_all'

$:.unshift(File.join(File.dirname(__FILE__)))
$:.unshift(File.join(File.dirname(__FILE__), 'support'))
$:.unshift(File.join(File.dirname(__FILE__), 'resources'))

require("./generated/zoo/support/picatrix.mason.pb")
require("./generated/zoo/support/mason")
require("./generated/zoo/support/zoo.pb")
require_all("generated/zoo/resources/*.rb")

class Zoo < Sinatra::Base
  include Picatrix::Mason

  set :public_folder => "public", :static => true

  get '/rels' do
    json({:@controls=>{:up=>"http://localhost:9292/", :self=>"http://localhost:9292/rels"}, :link_relations=>{:newest=>["root -(GET)-> animals", "animal -(GET)-> animals", "animals -(DELETE)-> animals"], :":item_id"=>["animals -(GET)-> animal"], :filter=>["animals -(GET)-> animals"], :create=>["animals -(POST)-> animal"], :edit=>["animal -(PATCH)-> animal"]}})
  end

  get '/' do
    status, headers, body = call env.merge("PATH_INFO" => "/animals")
    [status, headers, body]
  end

  def json(hash = {})
    # anything set here will be sent in all responses
    super({"@namespaces"=>{"z"=>{"name"=>"http://localhost:9292/rels#"}}}.merge!(hash))
  end


get "/animals" do

     response_body = {"animals" => AnimalsResource.as_hashes}

     controls = {}


     controls.merge!(send("create".to_sym, Animals).call("animals".pluralize, params))

     controls.merge!(send("filter".to_sym, Animals).call("animals".pluralize, params))

     controls.merge!(send("mason_self".to_sym, Animals).call("animals".pluralize, params))

     controls.merge!(send("mason_up".to_sym, Animals).call("animals".pluralize, params))


     json({
       :@controls => controls
     }.merge!(response_body))
   end

get "/animals/:item_id" do

     response_body = AnimalsResource.find(params[:item_id]).to_hash.except(:controls)

     controls = {}


     controls.merge!(send("edit".to_sym, Animal).call("animal".pluralize, params))

     controls.merge!(send("remove".to_sym, Animal).call("animal".pluralize, params))

     controls.merge!(send("mason_self".to_sym, Animal).call("animal".pluralize, params))

     controls.merge!(send("mason_up".to_sym, Animal).call("animal".pluralize, params))


     json({
       :@controls => controls
     }.merge!(response_body))
   end

post "/animals" do

     response_body = AnimalsResource.create(params[:item_id]).to_hash.except(:controls)

     controls = {}

     json({
       :@controls => controls
     }.merge!(response_body))
   end

delete "/animals/:item_id" do

     response_body = {"animals" => AnimalsResource.as_hashes}

     controls = {}

     json({
       :@controls => controls
     }.merge!(response_body))
   end

patch "/animals/:item_id" do

     response_body = AnimalsResource.find(params[:item_id]).to_hash.except(:controls)

     controls = {}


     controls.merge!(send("mason_self".to_sym, Animal).call("animal".pluralize, params))

     controls.merge!(send("mason_up".to_sym, Animal).call("animal".pluralize, params))


     json({
       :@controls => controls
     }.merge!(response_body))
   end

end
