module Picatrix
  class Buffy
    include Thor::Base
    include Thor::Actions

    attr_accessor :resources, :link_relations, :representation_klasses,
      :app_name

    # must be disjoint due to nature of protobuf
    ITEM_ACTIONS = %w{ Edit Remove }
    COL_ACTIONS = %w{ Create Filter }
    CRUFD = %w{ Create Edit Remove Filter }

    argument :name
    source_root File.dirname(__FILE__)

    def initialize(pacman, options = {}, app_name = "app")
      # thor related
      @options = options
      @destination_stack = [self.class.source_root]
      @app_name = app_name

      @representation_klasses = pacman.nodes.keys.map(&:camelize)

      @item_representations = @representation_klasses.map(&:singularize).uniq
      @col_representations = @representation_klasses.map(&:pluralize).uniq
      @resources = @item_representations + @col_representations

      @link_relations = pacman.edges.collect do |edge|
        target = edge[edge.keys.first][:target]
        link_relation = edge[edge.keys.first][:link_relation]
        if link_relation[0] != ':' && !CRUFD.map(&:downcase).include?(link_relation)
          "#{link_relation.camelize}" + "#{target.camelize}"
        end
      end.compact.uniq
    end

    def slay
      # TODO need to make prefix setter
      @prefix = app_name[0]
      @host = "http://localhost:9292/"

      template(
        'templates/namespaces.json',
        File.join("../../generated/#{app_name}/support/namespaces.json")
      )

      template(
        'templates/app.proto',
        File.join("../../generated/#{app_name}/support/#{app_name}.proto")
      )
    end
  end
end
