module Picatrix
  class Cruddy
    attr_accessor :controls

    # iow if it is addressable it has a self
    DEFAULT_CONTROLS = [:mason_self, :mason_up]

    def initialize(edges)
      @controls = edges.inject({}) do |memo, edge|
        source = edge.keys.first
        target = edge[source][:target]

        next memo if source.upcase == "ROOT"

        memo[target] = appropriate_controls_for(target)
        memo
      end
    end

    private
    def appropriate_controls_for(source)
      by_size = collection?(source) ? :collection : :item
      {
        source => template_map(by_size)
      }
    end
    def template_map(type)
      {
        # what controls do you want to get back
        # from a given method on a item/collection
        item: {
          get: [:edit, :remove] + DEFAULT_CONTROLS,
          patch: DEFAULT_CONTROLS,
          put: DEFAULT_CONTROLS,
          delete: [:mason_up]
        },
        collection: {
          get: [:create, :filter] + DEFAULT_CONTROLS,
          post: [:mason_self]
        }
      }[type]
    end
    def collection?(source)
      source.pluralize == source
    end
  end
end
