module Picatrix
  class DigraphParser < Parslet::Parser
    rule(:space) { str(" ") }
    rule(:indent) { str(" ").repeat(2) }
    rule(:newline) { str("\n") }
    rule(:eof) { any.absent? }
    rule(:comment) do
      indent.repeat(0) >>
      str("//") >>
      (newline.absent? >> any).repeat(1) >>
      newline
    end
    rule(:graph_setting) do
      indent >>
      (str(";").absent? >> any).repeat(0) >>
      str(";") >>
      newline
    end
    rule(:default_definitions) do
      graph_setting.repeat(0) >>
      indent >>
      str("edge [") >>
      (str("]").absent? >> any).repeat(0) >>
      str("]") >>
      newline
    end
    rule(:arrow) { str("->") }
    rule(:snake_case_key) do
      match('[a-zA-Z0-9_]').repeat(1)
    end
    rule(:quoted_string) do
      str('"') >>
      (
        (str('\\') >> any) | (str('"').absent? >> any)
      ).repeat(1) >>
      str('"')
    end
    rule(:style) do
      str("style=dotted") |
        str("style=bold") |
        delete_marker
    end
    rule(:post_marker) do
      str("arrowhead=normal")
    end
    rule(:delete_marker) do
      str("color=red fontcolor=red")
    end
    rule(:shape) do
      str("shape=note") |
        str("shape=folder") |
        str("shape=rectangle")
    end
    rule(:label) do
      str("[") >>
      str("label=") >>
      quoted_string.as(:label) >>
      space.repeat(0) >>
      style.repeat(0).as(:style) >>
      space.repeat(0) >>
      shape.repeat(0).as(:shape) >>
      str("]")
    end
    rule(:link_relation) do
      str("[") >>
      str("label=") >>
      quoted_string.as(:link_relation) >>
      space.repeat(0) >>
      style.repeat(0).as(:style) >>
      space.repeat(0) >>
      shape.repeat(0).as(:shape) >>
      space.repeat(0) >>
      post_marker.repeat(0).as(:post) >>
      str("]")
    end
    rule(:edge_target) do
      str("{") >>
      space.repeat(0) >>
      snake_case_key >>
      space.repeat(0) >>
      str("}")
    end
    rule(:root_line) do
      indent >>
      str("root") >>
      space >>
      (newline.absent? >> any).repeat(0) >>
      newline
    end
    rule(:digraph_open_tag) { str("digraph d {") }
    rule(:digraph_close_tag) { str("}") >> newline.repeat(0) >> eof }
    rule(:digraph_heading) do
      digraph_open_tag >>
      newline >>
      comment.repeat(0) >>
      default_definitions >>
      newline >>
      comment.repeat(0) >>
      root_of_api
    end
    rule(:root_of_api) do
      root_line >>
      (
        digraph_close_tag.absent? >>
        (
          comment |
          node_definition.as(:node) |
          edge_definition.as(:edge)
        ) >>
        newline.repeat(0)
      ).repeat >>
      digraph_close_tag
    end
    rule(:node_definition) do
      indent >>
      snake_case_key.as(:name) >>
      space.repeat(0) >>
      label >>
      newline
    end
    rule(:edge_definition) do
      indent >>
      snake_case_key.as(:outbound) >>
      space.repeat(0) >>
      arrow >>
      space.repeat(0) >>
      edge_target.as(:inbound) >>
      space.repeat(0) >>
      link_relation >>
      newline
    end
    root(:digraph_heading)
  end

end
