require 'require_all'
require 'json'

module Picatrix
  class Jenny
    include Thor::Base
    include Thor::Actions

    attr_accessor :app_name, :routes, :root_path, :rels,
      :destination_stack, :options

    argument :name
    source_root File.dirname(__FILE__)

    def initialize(pacman, options = {}, app_name = "app")
      # thor related
      @options = options
      @destination_stack = [self.class.source_root]

      @nodes = pacman.nodes
      @edges = pacman.edges

      @app_name = app_name
      @rels = LinkRelationIndex.new(@edges).masoned
      # TODO
      @meta = {}

      if File.exist?(
          namespaces_path = "generated/#{app_name}/support/namespaces.json"
      ) #TODO this should be coming in from graph instead
        @namespaces = JSON.parse(
          File.read(
            namespaces_path
          )
        )
        @prefix = "#{@namespaces["@namespaces"].keys.first}:"
      else
        puts "no generated/#{app_name}/support/namespaces.json file"
        @namespaces = {}
        @prefix = ""
      end
    end

    def just_do_it
      setup_framework_for(app_name)
      generate_sinatra_of(app_name)
    end

    def setup_framework_for(app_name)
      template(
        'templates/mason.rb',
        File.join("../../generated/#{app_name}/support/mason.rb")
      )

      Protoc.to_ruby(app_name)

      begin
        jenny_path = File.dirname( __FILE__ )
        $:.unshift File.expand_path( File.join( jenny_path, "..", "..", "generated" ) )
        $:.unshift File.expand_path( File.join( jenny_path, "..", "..", "generated", "#{app_name}" ) )
        $:.unshift File.expand_path( File.join( jenny_path, "..", "..", "generated", "#{app_name}", "support" ) )

        require_all("./generated/#{app_name}/**/*.rb")
      rescue LoadError => e
        puts "#{e}"
      end

      @control_templates = Cruddy.new(@edges).controls
      @resources = Prototyper.new(@nodes, {}, app_name).resources
      @routes = Routes.new(@edges, @control_templates)
      # make this available to the app scope
      @root_path = @routes.root_path
    end

    def generate_sinatra_of(app_name)
      template(
        'templates/app.rb',
        File.join("../../generated/#{app_name}/#{app_name}.rb")
      )
    end

    private
    def send_method_for(method)
      {
        post: :create,
        patch: :find,
        get: :find,
        delete: :find
      }[method]
    end
  end
end
